#!/usr/bin/env python
from unittest import TestCase
from acme_collectors.utils.constants import NOTIFICATION_TEMPLATE
from uuid import uuid4
import os

class TestNotificationGeneration(TestCase):
    def test_file_generation(self):
        email: str = "admin@test.com"
        subject: str = "TEST TEMPLATE GENERATION".title()
        server_name: str = "admin_node"
        server_address: str = "127.0.0.1"
        message: str = "This is a test template"
        current_file: str = f"{str(uuid4())}.log"

        notification = NOTIFICATION_TEMPLATE % (email,
                                                subject,
                                                server_name,
                                                server_address,
                                                current_file,
                                                message)

        with open(current_file, 'w') as f:
            f.write(notification)
        f.close()

        self.assertEqual(os.path.exists(current_file) ,True)



#!/usr/bin/env python
from acme_collectors.utils.helpers import get_current_credential_paths
from typing import Dict, List 
"""
Name: Rio Atmadja
Date: November 25, 2020
Description: Constant script
"""

# Create justpasteit_posts 
CREATE_TABLE: str = """
    CREATE TABLE `justpasteit_posts` (
        `post_id` INT(11) NOT NULL AUTO_INCREMENT, 
        `date_created` TEXT, 
        `post_modified` TEXT, 
        `number_views` VARCHAR(255), 
        `url` VARCHAR(255),
        `search_keyword` TEXT,  
        `image_url` TEXT, 
        `language` VARCHAR(255), 
        `posts` LONGTEXT , 
        PRIMARY KEY (post_id) 
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
"""

# Notification template
NOTIFICATION_TEMPLATE: str = """
To: %s
From: dscnotifier@gmail.com  
Subject: %s

NOTIFICATION 
--------------------------------
Server Name: %s
Server Address: %s
LOG Reference: %s
--------------------------------

MESSAGE
--------------------------------
%s
"""

# Create translated_posts 
TRANSLATED_TBL: str = """
    CREATE TABLE `translated_posts` (
        `translated_id` INT(11) NOT NULL AUTO_INCREMENT,
        `post_id` INT(11),
        `image_id` INT(11),
        `post_url` TEXT,
        `image_url` TEXT,
        `language` VARCHAR(255),
        `posts_md5` VARCHAR(255), 
        `translated` BOOLEAN, 
        PRIMARY KEY (translated_id) 
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8; 
"""
# Query to store images
IMAGE_URLS: str = """
    CREATE TABLE `image_urls_justpasteit` (
        `image_id` INT(11) NOT NULL AUTO_INCREMENT , 
        `post_url` VARCHAR(255), 
        `image_url` TEXT , 
        `embedded_text` LONGTEXT,
        `translated_text` LONGTEXT, 
        `language` VARCHAR(255), 
        `translated` BOOLEAN, 
        PRIMARY KEY (image_id) 
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
"""

# NOTE: Current topics for now, add more later
TOPICS: Dict = {
    'islamic_state': '%D8%A7%D9%84%D8%AF%D9%88%D9%84%D8%A9%20%D8%A7%D9%84%D8%A5%D8%B3%D9%84%D8%A7%D9%85%D9%8A%D8%A9',
    'daesh': '%D8%AF%D8%A7%D8%B9%D8%B4',
    'dogs_of_fire': '%D9%83%D9%84%D8%A7%D8%A8%20%D8%A7%D9%84%D9%86%D8%A7%D8%B1', # dogs of fire (term for a suicide bomber)
    'dogs_of_baghdadi': '%D9%83%D9%84%D8%A7%D8%A8%20%D8%A7%D9%84%D8%A8%D8%BA%D8%AF%D8%A7%D8%AF%D9%8A',
    'in_the_shadow_of_caliphate': '%D9%81%D9%8A%20%D8%B8%D9%84%20%D8%A7%D9%84%D8%AE%D9%84%D8%A7%D9%81%D8%A9', # word association with the crime/action commited by the daesh
    'mujahideen': '%D8%A7%D9%84%D9%85%D8%AC%D8%A7%D9%87%D8%AF%D9%88%D9%86',
    'lions_of_the_islamic_state': '%D8%A3%D8%B3%D9%88%D8%AF%20%D8%A7%D9%84%D8%AF%D9%88%D9%84%D8%A9%20%D8%A7%D9%84%D8%A5%D8%B3%D9%84%D8%A7%D9%85%D9%8A%D8%A9',
    'soldiers_of_the_caliphate': '%D8%AC%D9%86%D9%88%D8%AF%20%D8%A7%D9%84%D8%AE%D9%84%D8%A7%D9%81%D8%A9',
    'caliph': '%D8%A7%D9%84%D8%AE%D9%84%D8%A7%D9%81%D8%A9%20'

}

PKL_CLF: str = "./acme_collectors/utils/tools/random_forest.gz"
TONES: List[str] = ['Analytical', 'Joy', 'Sadness', 'Fear', 'Confident', 'Anger', 'Tentative'] 
HEADERS: Dict = {'Accept': '*/*',
                  'Accept-Language': 'en-US,en;q=0.5', 
                  'Encoding': 'gzip, deflate, br', 
                  'User-Agent': 'Mozilla/5.0 (ACME Content Collector; Linux x86_64; x64; rv:84.0) Gecko/20000101 Firefox/34.0'
                }
# NOTE: Location to your credentials
CREDENTIAL_PATH: str = get_current_credential_paths()



#!/usr/bin/env python
from unittest import TestCase
import pytest
import os
from acme_collectors.engines.cnn_engine import CNNENGINE
from typing import List

class TestCNNEngine(TestCase):

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master', reason='Local Test' )
    def test_cnn_links(self):
        cnn_engine = CNNENGINE(topic='Islamic State')
        links, pages = cnn_engine.get_cnn_links()
        return self.assertListEqual([len(links), len(pages)], [10, 5])


    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master', reason='Local Test')
    def test_cnn_parse_content(self):
        cnn_engine = CNNENGINE()
        current_article: str = cnn_engine.parse_content(url='https://www.cnn.com/2020/09/02/politics/american-isis-member-kuzu-pleads-guilty/index.html')

        article_title: str = "American ISIS member pleads guilty to supporting terror group"
        return self.assertEqual(article_title in current_article, True) 



#!/usr/bin/env python
from google.cloud import translate_v2 as translate
from google.cloud import vision
from acme_collectors.utils.helpers import load_credentials
import os

# Alias
from typing import Dict, List, Union

"""
Name: Rio Atmadja
Date: November 27, 2020 
Description: Google helper for natural language processing 
"""
class GoogleNLP(object):

    def __init__(self, creds_path: str):
        """
        NAME
            GoogleNLP

        DESCRIPTION
            Python module for ACME project for dealing with the natural language processing

        PACKAGE CONTENTS
            detect_original_language
            translate_to_language
            extract_text_from_image
        """

        if not os.path.exists(creds_path):
            raise FileNotFoundError("Please provide the right Google credentials")

        os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = creds_path
        self.cred_path: str = creds_path
        self.translate_client = translate.Client()
        self.image_client = vision.ImageAnnotatorClient()
        self.image_vision = vision.Image()


    def detect_original_language(self, sentence: str) -> Dict:
        """
        Description
        ------------
        Helper function to detect the given language of the given sentence

        Parameters
        ----------
        :params sentence: given a non-empty sentence

        Returns
        -------
        :return: a dictionary with language, confidence, and input attributes
        """

        if not sentence:
            raise AttributeError("Please provide the required parameter: sentence.")

        return self.translate_client.detect_language(sentence)

    def translate_to_language(self, sentence: str, language: str = 'en') -> Union[str, list]:
        """
        Description
        -----------
        Helper function to translate the given sentence to English, unless specified.

        Parameters
        ----------
        :params sentence: given a non-empty sentence
        :params language: optional parameters with default as English

        Returns
        --------
        :return: a dictionary with translated text, detected source language, and the input attributes
        """

        if not sentence:
            raise AttributeError("Please provide the required parameter: sentence.")

        return self.translate_client.translate(sentence, target_language=language)

    def extract_text_from_image(self, image_url: str) -> List:
        """
        Description
        -----------
        Helper function to extract words from the given image

        Parameters
        ----------
        :param image_url: given a valid image_url

        Returns
        -------
        :return: a list of words
        """

        self.image_vision.source.image_uri = image_url
        response = self.image_client.text_detection(image=self.image_vision)
        texts = response.text_annotations

        if not texts:
            return []

        return [text.description for text in texts]




#!/usr/bin/env python
from sklearn.ensemble import RandomForestClassifier
from sklearn.feature_extraction.text import CountVectorizer, TfidfTransformer 
import joblib 
from nltk import sent_tokenize 
import os 
import pandas as pd 
from acme_collectors.utils.constants import PKL_CLF, TONES 

from typing import Dict, List
from pandas.core.frame import DataFrame
"""
Name: Rio Atmadja
Date: January 08,2020 
Description: tone classifier utilities 
"""

def classify_tones(text, clf_path: str = PKL_CLF) -> Dict : 
    """
    Description 
    ------------

    Parameters
    -----------
    :text: given a non-empty text 

    Return
    ------
    :return: a dictionary with analytical, joy, sadness, fear, confident, anger, and tentative attributes 
    """ 
    if not text: 
        raise ValueError(f"You must provide a non-empty text.")

    if not os.path.exists(clf_path): 
        raise FileNotFoundError(f"Unable to find the following file : {PKL_CLF}. Please check for this file. {os.getcwd()}")

    # unpack pickle classifier here 
    pkl = joblib.load(clf_path)
    clf, tfidf, vect = tuple(pkl.values())

    mapped_tones: Dict = dict(zip(range(7), TONES))
    results: List = []
    for index,word in enumerate(sent_tokenize(text),1): 
        sentence_tone: str = mapped_tones.get(clf.predict(tfidf.transform(vect.transform([word]))).tolist()[0]) 
        tone_proba_score: DataFrame = pd.DataFrame(clf.predict_proba(tfidf.transform(vect.transform([word]))), 
                                               columns=TONES, 
                                               index=['Sentence']).transpose().sort_values(by='Sentence', ascending=False)

        results.append({'sentence': word,
                        'tone': sentence_tone,
                        'score': tone_proba_score.iloc[0][0] 
                       })
    return results



#!/usr/bin/env python
import pytest 
import os 
from unittest import TestCase 
from acme_collectors.engines.ibm_watson_nlp import WatsonNLP
"""
Name: Rio Atmadja
Date: December 05, 2020
Description: Test case for Watson Tone Analyzer 
"""
class TestWatsonToneAnalyzer(TestCase): 

    @pytest.mark.skipif(os.getenv("BITBUCKET_BRANCH") == "master" or os.getenv("BITBUCKET_BRANCH") == "develop",  
                      reason="Local Test Only.") 
    def test_tone_analyzer(self): 
        article: str = """
        In a statement, United Nations Secretary-General Antonio Guterres’s spokesman urged for “restraint and the need to avoid any actions that could lead to an escalation of tensions in the region”.
        But Iran’s Supreme Leader Ali Hosseini Khamenei called for the killers to be brought to justice, and a sense of discomfort has settled over some Iraqis.
        The killing of one of Iran’s towering figures felt reminiscent of the assassination of top Iranian General Qassem Soleimani in January.
        The killing of Soleimani by a US air strike paved the path towards an uptick in violence between American troops and Iran-backed armed groups, like Kataib Hezbollah.
        The group released a statement following the killing of Fakhrizadeh calling for revenge against the “Zionist-American-Saudi axis’ criminal record. The cost of their crimes must be high.”
        """ 
        tone_analyzer = WatsonNLP()
        return self.assertEqual(len(tone_analyzer.get_tone(sentence=article)), 3 )



#!/usr/bin/env python
from bs4 import BeautifulSoup
from urllib import request
import time
import re
"""
Name: Rio Atmadja
Date: November 25, 2020
Description: Content collectors to retrieve data from different streams such as website, and Apis
"""

# Alias
from typing import Dict, List

class ContentsCollector(object):
    """
    NAME
        ContentsCollectors

    DESCRIPTION
        Parse contents from the following sources:
            - Google
            - Just Paste it
            - Twitter
            - Aljazeera

    PACKAGE CONTENTS
        search_contents_from_google
        justpasteit_contents
    """

    def __init__(self):
        self.headers: Dict = {
            'user-agent': 'Mozilla/3.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/137.36 (KHTML, like Gecko) Chrome/56.0.4240.183 Safari/137.36',
            'accept': '*/*'
        }

    def search_contents_from_google(self, keyword: str, wait_time: int = 5) -> List[str]:
        """
        Description
        -----------
        This function uses Google search to find the contents from https://justpaste.it and return the links

        Parameters
        ----------
        :param keyword: given the valid keyword
        :param wait_time: given the optional parameter with the default value of 5 seconds

        Return
        ------
        :return: a list of links from https://justpaste.it
        """

        if not keyword:
            raise AttributeError("Please provide a keyword")

        try:
            google_url: str = f"https://www.google.com/search?num=100&start=10&hl=en&meta=&q=site%3Ajustpaste.it%20intext%3A({keyword})"
            response = request.urlopen(request.Request(url=google_url, headers=self.headers)).read()

            soup = BeautifulSoup(response, 'html.parser')
            results: List[str] = list(set(list(map(lambda tag: re.sub(".*.?q=|&sa=.*.", "", tag),
                                                   filter(lambda tag: 'https://justpaste.it' in str(tag),
                                                          [tag.find('a').get('href') for tag in soup.find_all('div') if
                                                           tag.find('a')])))))

            time.sleep(wait_time)
            return list(filter(lambda url: 'https://justpaste.it' in url, results))

        except ConnectionError as e:
            raise ConnectionError("Unable to make query to https://www.google.com. Please try again") from e

    def justpasteit_contents(self, url: str, search_keyword: str) -> Dict:
        """
        Description
        -----------
        Helper function to grab the website content from https://justpaste.it

        Parameters
        ----------
        :param url: given a valid url

        Returns
        -------
        :return:
        """

        if not all([url, search_keyword]):
            raise ValueError("Error: url and search_keyword are required parameters.")

        try:
            response = request.urlopen(request.Request(url=url, headers=self.headers)).read()
            soup = BeautifulSoup(response, 'html.parser')
            if not soup:
                return {'date_created': None,
                        'post_modified': None,
                        'number_views': None,
                        'url': None,
                        'search_keyword': None,
                        'image_url': None,
                        'language': None,
                        'posts': None
                        }

            javascript_elements: List = soup.find('head').find_all('script')
            for script in javascript_elements:
                if isinstance(script.string, str):
                    metadata: Dict = eval(
                        re.sub(r".*.window\.barOptions =", "", script.string.strip(' ').replace('true', 'True').replace('false','False').replace('null', 'None').split('\n')[-2]).strip(
                            ";").lstrip(' '))

            return {'date_created': metadata.get('createdText', None),
                    'post_modified': metadata.get('modifiedText', None),
                    'number_views': metadata.get('viewsText', None),
                    'url': url,
                    'search_keyword': search_keyword, 
                    'image_url': [img.get('src') for img in soup.find('body').find_all('img') if 'justpaste' in img.get('src') and '/img/r/' in img.get('src')],
                    'language': metadata.get('contentLang', None),
                    'posts': soup.find('body').text
                   }

        except ConnectionError as e:
            raise ConnectionError(f"Unable to make request to {url}. Please Try Again Later.")


#!/usr/bin/env python
import os
import requests
from uuid import uuid4
from subprocess import Popen, PIPE
from acme_collectors.utils.constants import NOTIFICATION_TEMPLATE
from acme_collectors.utils.helpers import saved_log

"""
Name: Rio Atmadja
Date: November 26, 2020
Description: Notification class to alert the user about the given events
"""

class Notifications(object):
    """
    NAME
        Notifications

    DESCRIPTION
        Python class to alert the users about the given event.

    PACKAGE CONTENTS
        send_notification
    """
    def __init__(self):
        self.ssmtp_path: str = "/usr/sbin/ssmtp"
        self.server_info = os.uname()
        self.server_address = requests.get('https://ifconfig.me').text
        self.log_path: str = os.path.join(os.path.expanduser("~/"), ".acme_log")

        if not os.path.exists(self.ssmtp_path):
            raise FileNotFoundError(
                f"Make sure to properly setup ssmtp on your {self.server_info.sysname} - {self.server_info.machine} - {self.server_info.release}")

    def send_notification(self, email_address: str, notification_subject: str, notification_msg: str):
        """
        Description
        -----------
        Helper function to send notification to the user about specific server events

        Parameters
        ----------
        :param email_address: given a valid email address
        :param notification_subject: given a valid notifaction subject
        :param notification_msg: given a valid notification message

        Returns
        -------
        :return:
        """
        if not all([email_address, notification_subject, notification_msg]):
            raise AttributeError("Please provide the following required parameters: email_address, "
                                 "notification_subject, and notification_msg")

        current_log: str = f"{str(uuid4())}.log"
        notification = NOTIFICATION_TEMPLATE % (email_address,
               notification_subject,
               self.server_info.nodename,
               self.server_address,
               current_log,
               notification_msg)

        if not os.path.exists(self.log_path):
            os.mkdir(self.log_path)

        # Saved the log to the given path
        log: str = os.path.join(self.log_path, current_log)
        saved_log(log=log, message=notification)

        try:
            Popen(f"{self.ssmtp_path} {email_address} < {log}", stderr=PIPE, stdout=PIPE, shell=True)

        except ConnectionError as e:
            saved_log(log=log, message="Something went wrong ...")
            raise ConnectionError(f"Unable to send notifiaction to {email_address}. Please try again later.")



#!/usr/bin/env python
from bs4 import BeautifulSoup 
from typing import Dict, List 
from urllib import request 
import re 

class AljazeeraEngine(object):
    """
    DESCRIPTION
    ------------
    Helper class responsible to retrieve contents from the Aljazeera website about a given topic. 

    PARAMETERS
    ----------
    :topic: given a valid topic 

    ATTRIBUTES
    -----------

    EXAMPLES
    --------- 
    ----
    """ 
    def __init__(self, topic: str = ''):
        if not topic: 
            raise ValueError("You must provide a valid topic.\n For example, AljazeeraEngine(topic='Islamic State')")

        self.topic: str = topic 
        self.headers: Dict = {'Accept': '*/*',
                            'Accept-Language': 'en-US,en;q=0.5', 
                            'Encoding': 'gzip, deflate, br', 
                            'User-Agent': 'Mozilla/5.0 (ACME Content Collector; Linux x86_64; x64; rv:84.0) Gecko/20000101 Firefox/34.0'
                            }


    def get_url(self) -> List[str]: 
        """
        Description
        -----------
        Helper funciton to generate query URLs 

        Return
        -------
        :return: a list of string with the aljazeera search query parameters 
        """
        non_alnum: List[str] = re.findall(r'\W',self.topic)
        encode_non_ascii: str = { char:hex(ord(char)).replace('0x', '%') for char in non_alnum} # encode non alpha numeric character 

        for key,value in encode_non_ascii.items(): 
            self.topic = self.topic.replace(key, value)

        return [ f"https://www.aljazeera.com/search/{self.topic}/page={page}" for page in range(1,10) ] 

    def aljazeera_browser(self, url: str) -> bytes: 
        """
        Description
        -----------
        A default browser to parse the Aljazeera contents 

        Parameters
        ----------
        :url: a required url parameter

        Return
        ------
        :return: a response from the Aljazeera website in bytes form 
        """
        if not url:
            raise ValueError(f"Please provide the right url. Please reffer to get_url method.")

        try: 
            return request.urlopen(request.Request(url=url,headers=self.headers)).read() 

        except ConnectionError as e: 
            raise ConnectionError(f"Error: Unable to parse the content from the given url {url}. Please check your query.") from e 

    def get_news_links(self) -> List[str]:
        """
        Description
        -----------
        Helper functions to get the valid Aljazeera news article links 

        Return
        -------
        :return: a list of Aljazeera links 
        """  
        aljazeera_links: List[str] = self.get_url()
        aljazeera_valid_links: List = []
        aljazeera_responses: List = []

        for url in aljazeera_links: 
            aljazeera_responses.append(self.aljazeera_browser(url=url)) 

        # extract the valid Aljazeera links 
        for response in aljazeera_responses:
            try: 
                # filter only Aljazeera links that contain unique article links 
                aljazeera_valid_links.extend(list(set([ tag.get('href') for tag in BeautifulSoup(response).find_all('a') if re.findall(r"/opinions/|/features/|/news/|/tag/", tag.get('href')) and 'www.aljazeera.com' in tag.get('href') ])))

            except: 
                raise Warning(f"Please check the following payload {response}")

        return  list(set(aljazeera_valid_links)) # make sure there is no duplicate links 

    def parse_contents(self) -> Dict:

        # parse the contents from Aljazeera
        article_links: List[str] = self.get_news_links()
        responses: Dict = {}
        for url in article_links: 
            current_page: bytes = self.aljazeera_browser(url=url)
            article_title: str = BeautifulSoup(current_page).find('h1').text.title() 
            responses[article_title] = '\n'.join([ p_tag.text for p_tag in BeautifulSoup(current_page).find_all('p') ]) 

        return responses
                



#!/usr/bin/python 
from ibm_watson import ToneAnalyzerV3 
from ibm_cloud_sdk_core.authenticators import IAMAuthenticator
from acme_collectors.utils.constants import CREDENTIAL_PATH 
from acme_collectors.utils.helpers import load_credentials
import os 
import json

from typing import Dict, List, Union

"""
                         ___ ____  __  __  __        ___  _____ ____   ___  _   _ 
                        |_ _| __ )|  \/  | \ \      / / \|_   _/ ___| / _ \| \ | |
                         | ||  _ \| |\/| |  \ \ /\ / / _ \ | | \___ \| | | |  \| |
                         | || |_) | |  | |   \ V  V / ___ \| |  ___) | |_| | |\  |
                        |___|____/|_|  |_|    \_/\_/_/   \_\_| |____/ \___/|_| \_|

                                                                                                                            
                                                 `-:.                                               
                                               -:++++/.                                             
                                             ./++///+++-`                                           
                                           ./+///+o+///++-`                                         
                                         .:+////+oys++//++/.                                        
                                       .:++//+++oyhyso+////+/.                                      
                                     .:+///+++ooyhhhyyso++////:.                                    
                                   .://///++ooyhhhhyhhyyo++////+:.                                  
                                 .://///++osyhhhhhhhhhhhyyso++///+:`                                
                               `-////++osyyhhhhhhhhhhhhhhhhyyso+////:.                              
                             `-/////+osyhhhhhhhhhhhhhhhhyhhhhyyyo+//++:`                            
                           `.////++++oosyyhhyyhhyhhhhhhyhhyhhyso+o+/////.                           
                          .://///o++++++oshhyhyhhhyhyhyyhhhys++ooo+o++///:`                         
                        `://////+o++o+o+o+yyyyyssyhhssysyhhy+++o++oo+++////-`                       
                      `-////++++ooo+o++oo+osoos++shyoo++osso++oo++ooooo+/////:`                     
                    `.////ooo++++o++oo+++++o++s++ohys+++o++o++oooooo+o+++o+////-`                   
                  `-:///+oyhs+++oo+ooo+oo++o++o++ohyso++oo+oo+++o+oo+oo+osys+////-`                 
                `-/////+oyyhhso+oo+oooooooooooo++shyso++oo+oo+ooo+oo+o++syhys++///+-`               
              .:/+///++oyyhhhhyysso+o+ooo+oo++o++syhsoo+oo+o++++++o++oyyhhhhyyo++//++:`             
           `.//++//++osyyhhhhhhhhysoo+oooooooosooyhhyyo+oo+oooooo++syhhhhhhhhyyso+///+/-`           
          -+++///+++syyhhhhhhhhhyy+oooooo+oo+oyyyhyyhyyyyo+oo++o++++yhhhhdhhhhhyys+o+////-`         
        -/++///++ssyyhhhhhhhhhhyyo+ooooooooosyhyooo+oosyys+o+ooo+++++yhhhdhhhhhhyyys++///+/-`       
      `/+///++osyhhyhhhhhhhdhhhyhyhhhyyhyyyhhhy++ooo+o+shhyyyyyyyyyyyyhhhhhhdhhhhhhyyyso+///+:`     
       -/+/+/++osyhhhhhhhhhdhhhyyyyyyyhhyyyhhhy++ooo+o+shhyyyyyyyyyyyyhhhhhhhhhhhhhhyyso++/+//.     
        `-/++//+oosyyhhhhhhhhhdhyo+oooooo++oshys+ooo+ooyhy+oooooooo++yhhhhhhhhhhhhyysso++++/:.      
          `-/+///++ooyyhhhhhhhhhhs++o+oooo+o+oyyyyyyyhyyho++oo+oo+o+ohhhhhhhhhhhyyoo+++//+/.        
            `:/+///+++oyhhhhhhhhhhs+o+o+o+oo++osyyyyhyssoo+ooooo+++oyhhhhhhhhhyyo+++/////.          
              `:+++/+o+osyhhhhyysso+o+oooo+so+oo+ohhyoooooo+ooooo++sysyyhhhhhysoo+++//:.            
                `:/+//+++syhyyo+oooooooooo+soooo+oyhyoo+oo++o+oo+ooo+o+osyyhyo++++++:.`             
                  `:////++yysoo+ooooooooo++oo+oo+oyhyoo+o++oooooo+o++++++yyho++/++/.`               
                    `://+/+o+oooooooooo+oo+so+oo+oyhys++ooooooooo+o++o+++oso+/+//-`                 
                      `:+///+ooo+ooooo+oo++ysooo+oyhyooooossoo+oo+oo++++++++++/-`                   
                        .:////oo+o+ooo++o+ohhyyssshhyssyyyyyo++o++oo++o+/+++/-`                     
                          .:/+/++++ooo++osyhhhhyyyyhyyyhhhhhyo+o+++o+++//+/-`                       
                           `./++//+++oossyhhhyyyyyyhyyyhhhhhhysoo+oo++/+/-`                         
                             `-//+//+osyyhhhyyyhhhyyyhhhhhyhhhyyy++/+//-`                           
                               `.:++/++osyyhhhyyhyyyyyhhhhhhyysoo//+/:`                             
                                  ./+//++oosyyhhhyhhyyhhhhysso+////:.                               
                                   `-////+ooosyhhhyhyhhyyyoo+//+//.                                 
                                     `-////+o++syhyhhyysoo+///+/-`                                  
                                       `-+/+/+oooyhhhyo++/+/+/-`                                    
                                         `:/+//+++yyso++//++:`                                      
                                           `:/+//+oo+/+/++:`                                        
                                             .:+//++//++/`                                          
                                               ./////+/-                                            
                                                 -///-`                                             
 `                                                 `                                                
                                                             
Name: Rio Atmadja
Date: December 04, 2020 
Description: IBM Watson Engine class to analyze a given tone from a sentence. 
                                                          
"""
class WatsonNLP(object):   

    @load_credentials(CREDENTIAL_PATH) 
    def __init__(self):
        """
        NAME 
            WatsonNLP 

        DESCRIPTION
            Python module for ACME project to analyze tone from a post or news paper article. 

        PACKAGE CONTENTS 
            get_tone
        """
        self.response: List = [] 
        self.ibm_tone_endpoint: str = os.getenv('IBM_TONE_ANALYZER_ENDPOINT')
        self.ibm_token_api: str = os.getenv('IBM_TOKEN_API')

        
    def get_tone(self, sentence:str, version: str = "2017-09-21") -> Union[List,Dict]:
        """
        Description
        -----------
        Helper function to extract tone from the given sentence 

        Parameters
        ----------
        :param sentence: given a non-empty string 
        :param version: an optional args with default of 2017-09-21 (Please reffer to the documentation https://cloud.ibm.com/apidocs/tone-analyzer?code=python) 

        Return
        ------
        :return: a list of dictionary that contains text, score, and tone name. 
        """
        ibm_tone_analyzer_endpoint: str = self.ibm_tone_endpoint
        if not sentence:
            raise ValueError("Please provide a valid sentence.")
        try:
            tone_analyzer = ToneAnalyzerV3(version=version, authenticator=IAMAuthenticator(self.ibm_token_api) ) 
            tone_analyzer.set_service_url(ibm_tone_analyzer_endpoint) 
            tones: Union[List, Dict] = tone_analyzer.tone(sentence).get_result().get('sentences_tone')

            if not tones:
                return [] 

            for sentence in tones: 
                sentence_tones: List = sentence.get('tones')
                if sentence_tones: 

                    score: float = sentence_tones[0].get('score') 
                    tone_name: str = sentence_tones[0].get('tone_name') 

                    self.response.append({'text': sentence.get('text'),
                                          'score': score,
                                          'tone_name': tone_name 
                                         })
                
            return self.response

        except ConnectionError as e:
            raise ConnectionError(f"Unable to connect to the following endpoint {self.ibm_tone_analyzer_endpoint}. Please try again later.") from e 



#!/usr/bin/env python
from acme_collectors.utils.helpers import label_article_pov, correct_spelling, get_organization_entities
import pytest 
from unittest import TestCase
from nltk.tree import Tree 
from typing import List 

class TestArticlePOV(TestCase):

    def test_pov(self): 
        current_article: str = """ 
        This man if the caliphate believes in him: Every Muslim dies a jaahilite death if he does not pledge allegiance to him !! On the other hand, 
        if this man was lost, corrupted, or infiltrated: he would be one of the most dangerous adversaries against the ummah in its history.
        """
        return self.assertEqual(label_article_pov(article=current_article), 'third_person')

    def test_correct_spelling(self): 
        return self.assertEqual(correct_spelling(article='dangerous zone'), 'dangerous zone' )  

    def test_get_organization_entities(self): 
        current_article: str = """
        This man if the caliphate believes in him: Every Muslim dies a jaahilite death if he does not pledge allegiance to him !! On the other hand, 
        if this man was lost, corrupted, or infiltrated: he would be one of the most dangerous adversaries against the ummah in its history.
        """ 
        organization_entity_tags: List[Tree] = [Tree('ORGANIZATION', [('Muslim', 'NNP')])]
        return self.assertEqual(get_organization_entities(article=current_article), organization_entity_tags)


#!/usr/bin/env python
from acme_collectors.utils.constants import HEADERS
import re
from phantomjs import Phantom
from typing import Dict, List
from bs4 import BeautifulSoup

class CNNENGINE(object):
    """
    DESCRIPTION
    -----------
    Helper class to parse the contents from the CNN class

    PARAMETERS
    ----------
    :topic: given a non-empty and valid topic
    :size: an optional parameters with a default search size of 10

    EXAMPLE
    -------
    >>>
    >>>
    """
    def __init__(self, topic: str =  "" , size: int = 10, category: str = ""):

        self.query: str = "" 
        self.config: Dict = {'headers': HEADERS}

        if topic:
            self.encoded_puncts: Dict = {char: hex(ord(char)).replace('0x', '%') for char in re.findall(r'\W', topic)}
            self.topic: str = ''.join([self.encoded_puncts.get(char, char) for char in topic])
            self.query = f"https://www.cnn.com/search?size={size}&q={self.topic.lower()}"

            if category:
                self.query = self.query + f"&category={category}"

            self.config['url'] = self.query 

    def cnn_browser(self) -> str:
        """
        Description
        -----------
        Helper method to make query to the CNN Website

        Parameters
        ----------
        :config: an optional dictionary parameter

        Returns
        -------
        :return: a string of response
        """
        if not self.config:
            raise AttributeError("Please provide a valid URL or kwargs configurations.")

        try:

            phantom_browser = Phantom()
            response: str = phantom_browser.download_page(conf=self.config)
            return response

        except ConnectionError as e:
            raise ConnectionError(f"[ERROR] Unable to connect to the following url: {self.query}") from e

    def get_categories(self) -> List[str]:
        """
        Description
        -----------
        Helper method to display different types of CNN categories

        Returns
        -------
        :return: a list with the category attributes
        """

        return ['us', 'world', 'politics', 'business', 'opinion', 'health', 'entertainment', 'style', 'travel']

    def get_cnn_links(self, url: str = "") -> tuple:
        """
        Description
        -----------
        Helper method to extract all the CNN's links

        Parameters
        ----------
        :url: an optional url parameter

        Returns
        -------
        :return: a list of strings that contain CNN links
        """
        
        if url:
            if 'www.cnn.com' not in url: 
                raise AttributeError(f"Please provide the right CNN URLs")

            self.config['url'] =  url
 
        cnn_response: str = self.cnn_browser()

        soup = BeautifulSoup(cnn_response.encode('utf-8'))
        cnn_links: [str] = list(set([f"https:{a.get('href')}" for a in soup.find_all('a') if re.findall(r"www.cnn.com.*.html", a.get('href')) and 'sitemap.html' not in a.get('href')]))
        next_page: List[str] = self.next_page(soup=soup)

        return (cnn_links, next_page)

    def parse_content(self, url: str) -> str:
        """
        Description
        -----------
        Helper method to parse the individual CNN page

        Parameters
        ----------
        :url: given a valid cnn link

        Returns
        -------
        :return: a string with the CNN contents
        """

        if not url and 'www.cnn.com' not in url: 
            raise AttributeError("Please provide a valid cnn url.")

        self.config['url'] =  url
        cnn_response: str = self.cnn_browser()
        soup = BeautifulSoup(cnn_response.encode('utf-8'))

        return ' '.join([x.text for x in soup.find_all('div') if 'l-container' in x.get('class', '')])

    def traverse_page(self, depth: int = 30) -> List:
        """
        Description
        -----------
        Helper function to traverse the CNN links by the given depth

        Parameters
        ----------
        :depth: an optional depth parameter with default of 30

        Returns
        -------
        :return: a list of the CNN articles
        """

        cnn_articles: List = []
        for page in range(0, depth, 10):
            url: str = f"{self.query}&from={page}&page={round(page/10) + 1}"
            cnn_links, next_page = self.get_cnn_links(url=url)

            if not next_page:
                return cnn_articles

            for link in cnn_links:
                link = re.sub(r"", "", link)
                cnn_articles.append(self.parse_content(url=link))

        return list(set(cnn_articles))  # return unique news articles 

    def next_page(self, soup: BeautifulSoup) -> List:
        """
        Description
        -----------
        Helper method to index the next page

        Parameters
        ----------
        :soup: a beautiful soup object

        Returns
        -------
        :return: a list of string that contains the page numbers
        """
        if not isinstance(soup, BeautifulSoup):
            raise TypeError("Please provide a beautiful soup object.")

        return [x.text for x in soup.find_all('span') if 'SearchPageLink' in ' '.join(x.get('class', []))]




#!/usr/bin/env python
import json
import os
from typing import List
from nltk import word_tokenize
import pandas as pd 
from typing import Dict 
from textblob import TextBlob
from nltk import ne_chunk , pos_tag, word_tokenize
from nltk.tree import Tree 
from typing import List 
from pandas.core.frame import DataFrame
import numpy as np 
"""
Name: Rio Atmadja
Date: November 27, 2020 
Description: Helper utilities for ACME Collectors 
"""
# Pronouns POV
FIRST_PERSON_PRP: List[str] = ['I', 'me', 'my', 'mine' 'myself']
SECOND_PERSON_PRP: List[str] = ['you', 'your', 'yours', 'yourself', 'yourselves']  
THIRD_PERSON_PRP: List[str] = ['he', 'him', 'his', 'himself', 'she', 'her', 'hers', 'herself', 'it', 'its', 'itself', 'they', 'them', 'their', 'theirs', 'themselves']


def load_credentials(credential_path: str = ""):
    """
    Description
    -----------
    Decorator to load credential_paths to the system environment 

    Parameters
    ----------
    :param credential_path: given a valid credential_path

    Returns
    -------
    :return: 
    """
    def wrap(function): 

        def wrap_function(*args, **kwargs): 
            
            # check if the credential_path exists 
            if not os.path.exists(credential_path):
                raise FileNotFoundError(f"Please provide the right credential_paths. Please check the following credential {credential_path}")
            
            # read the credential to the memory  
            creds: str = open(credential_path, 'r').read().strip("\n") 
            
            # before loading make sure that the string representation is in dictionary format 
            if not creds.index('{') == 0 and not creds.index('}') == len(creds) - 1: 
                raise TypeError(f"Please provide the right JSON format for the following file: {credential_path}") 

            # load the credential_paths to the system environments 
            for key,value in json.loads(creds).items(): 
                os.environ[key] = value # load all the credentials to the system variables (Credit: Google Cloud API behavior)  

            return function(*args, **kwargs) 

        return wrap_function 

    return wrap 
        

def saved_log(log: str, message: str):
    """
    Description
    -----------
    Helper function to write message to the given path

    Parameters
    ----------
    :param log: given a valid path
    :param message: given a valid message

    Returns
    -------
    :return:
    """
    if not all([log, message]):
        raise AttributeError("You must provide the required parameters log and message.")

    with open(log, 'w') as f:
        f.write(message)
    f.close()

def get_current_credential_paths(credential_path: str = "" , credential_file_name: str = "credentials.json") -> str :
    """
    Description
    -----------
    Helper function to return the credentials path. 
    Note: Temporary Work around to the static credential path and this version might be deprecated in the future. 
    
    Parameters
    ----------
    :credential_path: given an optional parameter with default of empty string
    :credential_file_name: given an optional parameter with default of credentials.json 
    
    Return
    -------
    :return: a path to the credentials
    """
    
    # if the user supply the credential path, then we're good
    if credential_path:
        return credential_path

    # Support Linux and Mac OSX Only at this version
    if not hasattr(os, 'uname'):
        raise NotImplementedError(f"This library supports only Mac OSX and Linux Only")

    passwd_file: str = "/etc/passwd"
    if not os.path.exists(passwd_file):
        raise FileNotFoundError(f"Unable to find {passwd_file}. That's odd, Are you running *.nix Operating System?")

    current_user: str = [attr.split(":")[0] for attr in open(passwd_file, 'r').read().strip().split("\n") if
                         str(os.getuid()) in str(attr.split(":")[2])][0]  # Mapped the current user

    return os.path.join(os.path.expanduser(f"~{current_user}"), credential_file_name)

def correct_spelling(article: str) -> str:
    """
    Description 
    -----------
    Helper function to correct spelling in the post

    Parameters
    ----------
    :article: given a non-empty article

    Return
    ------
    :return: an article with correct spelling
    """
    if not article: 
        raise ValueError("Please provide a text or an article.")

    return ' '.join( [ word for word in TextBlob(article).correct().words ] )

def label_article_pov(article: str) -> str:
    """
    Description
    -----------
    Helper function to label an article into first, second, and third POV. 

    Parameters
    ----------
    :article: given a non-empty article

    Return
    ------
    :return: a pov label 
    """
    if not article: 
        raise ValueError("Please provide a text or an article.")
        
    tokenize_articles: List[str] = word_tokenize(article)
    
    labels: Dict = {}
    for word in tokenize_articles:

        if word in FIRST_PERSON_PRP: 
            labels[word] = 'first_person'

        elif word in SECOND_PERSON_PRP: 
            labels[word] = 'second_person'

        elif word in THIRD_PERSON_PRP: 
            labels[word] = 'third_person'

        else: 
            labels[word] = 'third_person'

    results: DataFrame = pd.DataFrame(labels, index=['POV']).transpose()['POV'] 
    return results.value_counts().index[0]

def get_organization_entities(article: str) -> List[Tree]:
    """
    Description
    -----------
    Helper function to get the organization entities. 

    Parameters
    ----------
    :article: given a non-empty article

    Return
    ------
    :return: a list of organization entities 
    """
 
    if not article: 
        raise ValueError("Please provide a text or an article.")

    organization_entities: List = []
    for word in ne_chunk( pos_tag(word_tokenize(article))):
        if hasattr(word, 'label'):
            if word.label() == 'ORGANIZATION':
                organization_entities.append(word)

    return organization_entities



#!/usr/bin/python 
import os 
import pytest 
from typing import Dict,List 
from unittest import TestCase 
from acme_collectors.engines.aljazeera_engine import AljazeeraEngine 
class TestAljazeeraEngines(TestCase):

    @pytest.mark.skipif( os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master', reason='Local Test')
    def test_valid_urls(self):

        current_topic: str = "islamic state"
        response: List[str] = AljazeeraEngine(topic=current_topic).get_news_links() 
        return self.assertEqual(len(response), 12 )

    @pytest.mark.skipif( os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master', reason='Local Test')
    def test_parse_contents(self):

        current_topic: str = "islamic state"
        response: Dict = AljazeeraEngine(topic=current_topic).parse_contents() 
        aljazeera_topics: List[str] = list(response.keys())
        return self.assertEqual(len(aljazeera_topics), 12)


#!/usr/bin/env python 
from acme_collectors.utils.tone_classifiers import classify_tones 
from unittest import TestCase  
from typing import Dict, List 

article: str = """
The bombing of two schools and five hospitals in Syria allegedly by Russian warplanes has been slammed as a “war crime,” as the ongoing bloodshed diminished hopes for a cease-fire in the conflict.
Up to 50 civilians, including children, died when missiles hit at least five medical buildings and two schools in the Aleppo and Idlib provinces on Monday.
U.N. Secretary General Ban Ki-moon said the raids violated international law and “cast a shadow” over efforts to end Syria's five-year civil war. He did not say who was responsible for the attacks but groups monitoring the conflict suspected the strikes were carried out by Russia, an ally of the Syrian regime.
Russia’s Health Minister Veronika Skvortsova denied Moscow was responsible, saying its military had targeted Islamic State of Iraq and the Levant (ISIL) infrastructure and that she had no reason to believe it had bombed civilians.
“We are confident that [there is] no way it could be done by our defense forces. This contradicts our ideology,” she said.
French Foreign Minister Jean-Marc Ayrault said attacks on health facilities in Syria by the regime or its supporters were “unacceptable and must stop immediately.”
Turkey on Monday separately accused Russia of “an obvious war crime” and warned that there would be consequences if Russia did not immediately end such attacks.
Syria's ambassador to Moscow, Riad Haddad, said the hospital was targeted by a U.S. raid. “American warplanes destroyed it. Russian warplanes had nothing to do with any of it. The information that has been gathered will completely back that up,” he told Russian state television channel Rossiya 24.
The U.S., which, like the U.N., did not specify who carried out the strikes, said two civilian hospitals were hit in northern Syria: one run by medical charity Doctors Without Borders and another in rebel-held Azaz.
“That the Assad regime and its supporters would continue these attacks … casts doubt on Russia's willingness and/or ability to help bring to a stop the continued brutality of the Assad regime against its own people,” the U.S. State Department said.
Syrian President Bashar Assad warned Turkey and Saudi Arabia that any ground invasion of Syria would have “global repercussions” and said sending in troops would not be a “picnic.”
Commenting on an agreement brokered last week by the U.S., Russia and other world powers for a temporary cessation of hostilities, he said, “Cease-fires occur between armies and states but never between a state and terrorists.”
“They say that they want a cease-fire within a week. All right, who will talk to a terrorist organization if it refuses to cease-fire? Who will punish it?”
Assad's comments were his first since the agreement on Friday to bring about a pause in fighting within a week.
Earlier this month, Saudi Arabia and the UAE said they were prepared to send ground troops to Syria to fight ISIL if a U.S.-led coalition targeting the group with airstrikes agreed to the offer.
Russia warned that if foreign troops entered the country, it could lead to a world war.
"""
class TestClassifier(TestCase):

    def test_classify_tone(self):

        response: Dict = list(map(lambda tone: (tone.get('tone'), tone.get('score')) , classify_tones(text=article))) 
        result: List[float] =  [('Fear', 0.36),
                                ('Sadness', 0.96),
                                ('Confident', 0.43),
                                ('Analytical', 0.4493590748290803),
                                ('Tentative', 0.31),
                                ('Analytical', 0.63610357941074),
                                ('Analytical', 0.5353191489361702),
                                ('Analytical', 0.55475),
                                ('Sadness', 0.58),
                                ('Analytical', 0.6341059406498993),
                                ('Fear', 0.43),
                                ('Fear', 0.37),
                                ('Analytical', 0.71475),
                                ('Analytical', 0.6191721158958001),
                                ('Analytical', 0.33475),
                                ('Analytical', 0.6909432550238382),
                                ('Joy', 0.5407001192885269),
                                ('Analytical', 0.344594250053339),
                                ('Joy', 0.6555713378889204)
                               ]
        return self.assertListEqual(result, response) 


#!/usr/bin/env python
import pytest 
from unittest import TestCase
from acme_collectors.utils.pdf_parser import parse_pdf_text
import os 

class TestPDFUtils(TestCase):

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == "master" or os.getenv('BITBUCKET_BRANCH') == "develop", reason='Local Test')
    def test_pdf_extract(self):
        pdf_url: str = "https://justpaste.it/cosrtfis/pdf"
        extracted_document = parse_pdf_text(document_url=pdf_url) 
        return self.assertEqual(len(extracted_document), 10)

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == "master" or os.getenv('BITBUCKET_BRANCH') == "develop", reason='Local Test')
    def test_save_pdf_document(self): 
        pdf_url: str = "https://justpaste.it/cosrtfis/pdf"
        extracted_document = parse_pdf_text(document_url=pdf_url, file_name='local_test.pdf', save_pdf=True)
        print("EXTRACTED KEYS", extracted_document.keys() )
        file_name: str = extracted_document.get('file_name', None)
        
        self.assertEqual(os.path.exists(file_name), True)




#!/usr/bin/env python 
from acme_collectors.utils.constants import HEADERS
from pdfreader.viewer.simple import SimplePDFViewer 
from typing import Dict
from uuid import uuid4
import re
from urllib import request
import os
import ssl

def ACME_BROWSER(url: str) -> bytes:
    """
    Description
    -----------
    This is the current browser for the ACME project 

    Parameters
    ----------
    :url: given a valid url 

    Return
    ------
    :return: A http/https response, otherwise return the error message 
    """
    try:
        ctx = ssl.create_default_context()
        ctx.check_hostname = False
        ctx.verify_mode = ssl.CERT_NONE
        response: bytes = request.urlopen(request.Request(url=url, headers=HEADERS), context=ctx).read()
        return response 

    except ConnectionError as e: 
        raise ConnectionError(f"Unable to connect to the following URLs {url}") from e 

def parse_pdf_text(document_url: str, save_pdf: bool = False, file_name: str = str(uuid4())) -> Dict:
    """
    Description
    -----------
    Helper function to extract the text from the given document url. If the file is located remotely, this function will attempt to fetch the file and parse it.
    Otherwise, open the document and extract the text. 

    Paramters
    ---------
    :document_url: given a valid pdf URLs
    :save_pdf: an option argument with default set to False (To save the file)
    :file_name: an option argument with default set to UUID4, unless specified 

    Return
    ------
    :return: a dictionary with the extracted text and page numbers, if save_pdf is set to true, there is an additional attributes called file_name. 
    """ 
    # check if this is a URLS  
    is_url = re.match(r"^(www|http|[a-z]+).*.\.[a-z]{2,3}(/\w+){0,}", document_url.lower() )
    raw_document = None 
    extracted_text: Dict = {}

    if is_url: 
        raw_document = ACME_BROWSER(url=is_url.group())
    # if it's not a url link, attempt to open the local file 
    if not raw_document: 
        if not os.path.exists(document_url): 
            raise FileNotFoundError(f"[+] Unable to find the following file {document_url}")

        raw_document: bytes = open(document_url, 'rb').read() 

    pdf_contents: SimplePDFViewer = SimplePDFViewer(raw_document)
    while True:
        try:
            pdf_contents.render() 
            pdf_iterator = pdf_contents.iter_pages()
            extracted_text[pdf_iterator.viewer.current_page_number] = ''.join(pdf_iterator.viewer.canvas.strings) 
            pdf_iterator.viewer.next() 

        except:
            break 

    if save_pdf:
        with open(file_name, 'wt') as f: 
            for page_number, content in extracted_text.items():
                f.write(f"[{page_number}] {content}\n")
        f.close()

    return {**extracted_text, **{'file_name': file_name } } if save_pdf else extracted_text